import { animate, AUTO_STYLE, state, style, transition, trigger } from '@angular/animations';
import { Component, OnInit } from '@angular/core';
import { UntypedFormGroup } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { DateTime } from 'luxon';
import { structureFormStep } from '../../form/form-view/structure-form/structureFormStep.enum';
import { Owner } from '../../models/owner.model';
import { Structure } from '../../models/structure.model';
import { NotificationService } from '../../services/notification.service';
import { StructureService } from '../../services/structure.service';
import { ButtonType } from '../../shared/components/button/buttonType.enum';
import { Demarches } from '../../shared/enum/demarches.enum';
import { FreeWorkshop } from '../../shared/enum/freeWorkshop.enum';
import { AccessModality } from '../../structure-list/enum/access-modality.enum';
import { Equipment } from '../../structure-list/enum/equipment.enum';
import { Category } from '../../structure-list/models/category.model';
import { formUtils, IStructureSummary } from '../../utils/formUtils';
import { Utils } from '../../utils/utils';

@Component({
  selector: 'app-structure-edition-summary',
  templateUrl: './structure-edition-summary.component.html',
  styleUrls: ['./structure-edition-summary.component.scss'],
  animations: [
    trigger('show', [
      state('true', style({ height: AUTO_STYLE, visibility: AUTO_STYLE, margin: '8px 0' })),
      state('false', style({ height: '0px', visibility: 'hidden', margin: '0' })),
      transition('true => false', animate('300ms ease-out')),
      transition('false => true', animate('300ms ease-out')),
    ]),
  ],
})
export class StructureEditionSummaryComponent implements OnInit {
  public structure: Structure;
  public structureForm: UntypedFormGroup;
  public members: Owner[];
  public summary: IStructureSummary[] = new formUtils().structureSummary;
  public structureFormStep = structureFormStep;
  public buttonTypeEnum = ButtonType;
  public equipmentEnum = Equipment;

  //Digital learning
  public categories: Category[] = [];
  public showBaseSkills = false;
  public showAdvancedSkills = false;
  public mailHref: string;
  // set to true when last updated date of a structure is > 6 months
  public isUpdateStructure = false;

  constructor(
    private notificationService: NotificationService,
    private route: ActivatedRoute,
    private structureService: StructureService,
    private utils: Utils,
    public router: Router
  ) {}

  // There is a non-blocking error in console when open edit mode that occurs only in development mode, please refer https://angular.io/errors/NG0100 for more info

  ngOnInit(): void {
    this.route.data.subscribe((data) => {
      if (data.structure) {
        this.structure = new Structure(data.structure);
        this.structureForm = new formUtils().createStructureForm(this.structure, true);
        if (this.structure.contactMail) this.mailHref = `mailto:${this.structure.contactMail}`;

        const updatedAt = DateTime.fromISO(this.structure.updatedAt);
        const sixMonthsAgo = DateTime.local().minus({ month: 6 });
        if (updatedAt < sixMonthsAgo) this.isUpdateStructure = true;

        this.structureService
          .getStructureWithOwners(data.structure._id, null)
          .subscribe((result) => (this.members = result.owners));
      }
    });
  }

  public goBack(): void {
    history.back();
  }

  public goToEdit(step: structureFormStep): void {
    this.router.navigate(['/form/structure', this.structure._id, structureFormStep[step]]);
  }

  public isFormValid(): boolean {
    for (const field in this.structure) {
      if (!['_id', 'numero', 'createdAt', 'updatedAt'].includes(field)) {
        if (this.structureForm.get(field)?.valid === false) return false;
      }
    }
    return true;
  }

  public updateStructureUpdateDate(): void {
    this.structureService.editStructure({}, this.structure._id).subscribe(() => {
      this.notificationService.showSuccess('La structure a bien été mise à jour.', '');
      this.isUpdateStructure = false;
    });
  }

  public keepOriginalOrder = (a, _b) => a.key;

  public formatAddress(): string {
    let address = '';
    if (this.structure.address.numero) address += `${this.structure.address.numero} `;
    if (this.structure.address.street) address += `${this.structure.address.street}, `;
    if (this.structure.address.commune) address += `${this.structure.address.commune}`;
    return address;
  }

  public getAccessLabel(accessModality: AccessModality): string {
    return this.utils.getAccessLabel(accessModality);
  }

  public containsHours(): boolean {
    for (const day in this.structure.hours) {
      if (this.structure.hours[day].time.length) return true;
    }
    return false;
  }

  public isFieldValid(field: string, theme?: string): boolean {
    if (theme) {
      return this.structureForm.get(theme).get([field]).valid;
    }
    return this.structureForm.get([field]).valid;
  }

  public hasSocialNetworks(): boolean {
    return this.utils.hasSocialNetwork(this.structure);
  }

  public hasWebsite(): boolean {
    return this.utils.hasWebsite(this.structure);
  }

  public containsDigitalHelp(): boolean {
    return this.structure.categoriesDisplay.onlineProcedures?.length > 0;
  }

  public getDigitalHelpLabel(digitalHelp: Demarches): string {
    return this.utils.getDigitalHelpLabel(digitalHelp);
  }

  public isBaseSkills(): boolean {
    return this.structure.categories.baseSkills?.length > 0;
  }
  public isAdvancedSkills(): boolean {
    return this.structure.categories.advancedSkills?.length > 0;
  }

  public containsDigitalLearning(): boolean {
    return this.isBaseSkills() || this.isAdvancedSkills();
  }

  public toggleBaseSkills(): void {
    this.showBaseSkills = !this.showBaseSkills;
  }

  public toogleAdvancedSkills(): void {
    this.showAdvancedSkills = !this.showAdvancedSkills;
  }

  public getWorkshopPricingLabel(freeWorkshop: FreeWorkshop): string {
    return this.utils.getWorkshopPricingLabel(freeWorkshop);
  }

  public hasWifi(selfServiceMaterial: string[]) {
    return this.utils.hasWifi(selfServiceMaterial);
  }

  public hasEquipments(structure: Structure): boolean {
    return this.utils.hasEquipments(structure.nbComputers, structure.nbPrinters, structure.nbScanners);
  }
  public filterOnlyEquipments(selfServiceMaterial: string[]): string[] {
    return this.utils.filterOnlyEquipments(selfServiceMaterial);
  }
  public getEquipmentsLabelAndValue(equipment: Equipment, number: number): string {
    return this.utils.getEquipmentsLabelAndValue(equipment, number);
  }

  public goToManageMembers(): void {
    this.router.navigate([`/profile/structure-members-management/${this.structure._id}`]);
  }
}
