import { animate, AUTO_STYLE, state, style, transition, trigger } from '@angular/animations';
import { Location } from '@angular/common';
import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { ParametersService } from '../../../admin/services/parameters.service';
import { Owner } from '../../../models/owner.model';
import { Structure } from '../../../models/structure.model';
import { TclStopPoint } from '../../../models/tclStopPoint.model';
import { User } from '../../../models/user.model';
import { ProfileService } from '../../../profile/services/profile.service';
import { AuthService } from '../../../services/auth.service';
import { StructureService } from '../../../services/structure.service';
import { TclService } from '../../../services/tcl.service';
import { UserService } from '../../../services/user.service';
import { PrintService } from '../../../shared/service/print.service';
import { Utils } from '../../../utils/utils';
import { AccessModality } from '../../enum/access-modality.enum';
import { Equipment } from '../../enum/equipment.enum';
import { Category } from '../../models/category.model';
import { SearchService } from '../../services/search.service';
@Component({
  selector: 'app-structure-details',
  templateUrl: './structure-details.component.html',
  styleUrls: ['./structure-details.component.scss'],
  animations: [
    trigger('slideInOut', [
      transition(':enter', [style({ left: '-600px' }), animate('200ms ease-in', style({ left: '0' }))]),
      transition(':leave', [animate('200ms ease-in', style({ left: '-600px' }))]),
    ]),
    trigger('fadeInOut', [
      transition(':enter', [
        style({ backgroundColor: 'rgb(00, 00, 00, 0)' }),
        animate('200ms ease-in', style({ backgroundColor: 'rgb(00, 00, 00, 0.6)' })),
      ]),
      transition(':leave', [animate('200ms ease-in', style({ backgroundColor: 'rgb(00, 00, 00, 0)' }))]),
    ]),
    trigger('show', [
      state('true', style({ height: AUTO_STYLE, visibility: AUTO_STYLE, margin: '8px 0' })),
      state('false', style({ height: '0px', visibility: 'hidden', margin: '0' })),
      transition('true => false', animate('300ms ease-out')),
      transition('false => true', animate('300ms ease-out')),
    ]),
  ],
})
export class StructureDetailsComponent implements OnInit {
  @Input() public structure: Structure;
  @Output() public closeDetails: EventEmitter<boolean> = new EventEmitter<boolean>();
  public accessModality = AccessModality;
  public categories: Category[] = [];
  public showBaseSkills = false;
  public showAdvancedSkills = false;
  public tclStopPoints: TclStopPoint[] = [];
  public printMode = false;
  public isLoading = true;
  public lockdownInfoDisplay = false;
  public currentProfile: User = null;
  public deleteModalOpenned = false;
  public structureErrorModalOpenned = false;
  public claimModalOpenned: boolean;
  public joinModalOpenned = false;
  public pendingModalOpenned = false;
  public structureAdmins: Owner[] = [];
  public fullScreen = false;

  constructor(
    private printService: PrintService,
    private searchService: SearchService,
    private structureService: StructureService,
    private tclService: TclService,
    private profileService: ProfileService,
    private authService: AuthService,
    private parametersService: ParametersService,
    private route: ActivatedRoute,
    private location: Location,
    private router: Router,
    private usersService: UserService
  ) {
    this.route.url.subscribe((url) => {
      if (url.length > 0 && url[0].path === 'structure') {
        this.structure = new Structure(this.printService.structure);
        this.printMode = true;
        // Display formations for printing
        this.toggleAdvancedSkills();
        this.toggleBaseSkills();
        this.initForm();
      }
    });
    this.parametersService.getParameters().subscribe((params) => {
      this.lockdownInfoDisplay = params.lockdownInfoDisplay;
    });
  }

  async ngOnInit(): Promise<void> {
    this.route.queryParams.subscribe((queryParams) => {
      if (queryParams.id) {
        this.structureService.getStructure(queryParams.id).subscribe((structure) => {
          this.structure = new Structure(structure);
          this.initForm();
        });
      } else if (!this.printMode) {
        this.structure = null;
      }
    });
    this.route.data.subscribe((data) => {
      if (data.fullScreen) {
        this.fullScreen = true;
      }
    });
  }

  private async initForm(): Promise<void> {
    if (this.userIsLoggedIn()) {
      this.currentProfile = await this.profileService.getProfile();
      this.structureService.getStructureWithOwners(this.structure._id, this.currentProfile).subscribe((res) => {
        this.structureAdmins = res.owners;
      });
    }
    this.structure.isClaimed = await this.structureService
      .isClaimed(this.structure._id, this.currentProfile)
      .toPromise();

    // GetTclStopPoints
    this.getTclStopPoints();
    this.searchService.getCategories().subscribe((categories) => {
      this.categories = categories;
      this.setServiceCategories();
      if (this.printMode) {
        this.printService.onDataReady();
      }
      this.isLoading = false;
    });
  }

  public userIsLoggedIn(): boolean {
    return this.authService.isLoggedIn();
  }

  public getEquipmentsLabel(equipment: Equipment): string {
    switch (equipment) {
      case Equipment.wifi:
        return 'Wifi en accès libre';
      case Equipment.printer:
        return this.structure.nbPrinters > 1 ? 'Imprimantes' : 'Imprimante';
      case Equipment.computer:
        return this.structure.nbComputers > 1 ? 'Ordinateurs' : 'Ordinateur';
      case Equipment.scanner:
        return this.structure.nbScanners > 1 ? 'Scanners' : 'Scanner';
      default:
        return null;
    }
  }

  public close(): void {
    if (this.router.url.startsWith('/acteurs')) {
      this.router.navigateByUrl('/acteurs');
      return;
    }
    this.route.url.subscribe((urls) => {
      if (urls.length > 0 && urls[0].path !== 'orientation') {
        this.router.navigate(['/acteurs'], {
          relativeTo: this.route,
          queryParams: {
            id: null,
          },
          queryParamsHandling: 'merge',
        });
      } else {
        this.isLoading = true;
        this.location.back();
      }
    });
  }

  public print(): void {
    this.printService.printDocument('structure', this.structure);
  }

  public toggleDeleteModal(): void {
    this.deleteModalOpenned = !this.deleteModalOpenned;
  }

  public toggleClaimModal(): void {
    this.claimModalOpenned = !this.claimModalOpenned;
  }

  public toggleJoinModal(): void {
    this.joinModalOpenned = !this.joinModalOpenned;
  }

  public togglePendingModal(): void {
    this.pendingModalOpenned = !this.pendingModalOpenned;
  }

  public handleJoin(): void {
    if (this.userIsLoggedIn()) {
      if (this.structure.isClaimed) {
        if (this.profileService.isPendingLinkedToStructure(this.structure._id)) {
          this.togglePendingModal();
        } else {
          this.toggleJoinModal();
        }
      } else {
        this.toggleClaimModal();
      }
    } else {
      this.router.navigate(['login'], { queryParams: { returnUrl: `acteurs?id=${this.structure._id}` } });
    }
  }
  public handleModify(): void {
    this.router.navigateByUrl(`/profile/edit-structure/${this.structure._id}`);
  }

  public deleteStructure(shouldDelete: boolean): void {
    this.toggleDeleteModal();
    if (shouldDelete) {
      this.structureService.delete(this.structure._id).subscribe((res) => {
        this.reload();
      });
    }
  }

  private reload(): void {
    this.router.routeReuseStrategy.shouldReuseRoute = () => false;
    this.router.onSameUrlNavigation = 'reload';
    this.router.navigate(['./'], { relativeTo: this.route });
  }

  public claimStructure(shouldClaim: boolean): void {
    this.toggleClaimModal();
    if (shouldClaim) {
      this.structureService
        .claimStructureWithAccount(this.structure._id, this.authService.userValue.username)
        .subscribe();
      this.router.navigate(['join-request', this.structure._id], { state: { isClaimed: this.structure.isClaimed } });
    }
  }

  public joinStructure(shouldJoin: boolean): void {
    this.toggleJoinModal();
    if (shouldJoin) {
      this.usersService.joinStructure(this.structure._id, this.authService.userValue.username).subscribe();
      this.router.navigate(['join-request', this.structure._id], { state: { isClaimed: this.structure.isClaimed } });
    }
  }

  /**
   * Map categories ids to there real names
   */
  public setServiceCategories(): void {
    this.categories.forEach((category) => {
      const structureModuleIds = this.structure.categories[category.id];
      if (structureModuleIds) {
        const moduleNames = category.modules
          .map((module) => {
            if (structureModuleIds.includes(module.id)) {
              return module.name;
            }
          })
          .filter((value) => value !== undefined)
          .filter((value) => value !== 'Autres'); // Filter other categories
        this.structure.categoriesDisplay = {
          ...this.structure.categoriesDisplay,
          [category.id]: moduleNames,
        };
      }
    });
  }

  public keepOriginalOrder = (a, b) => a.key;

  public hasBaseSkills(): boolean {
    return this.structure.categories.baseSkills?.length > 0;
  }
  public hasAdvancedSkills(): boolean {
    return this.structure.categories.advancedSkills?.length > 0;
  }

  public getTclStopPoints(): void {
    this.tclService.getTclStopPointBycoord(this.structure.getLon(), this.structure.getLat()).subscribe((res) => {
      this.tclStopPoints = res;
    });
  }

  public displayModalError(): void {
    this.structureErrorModalOpenned = !this.structureErrorModalOpenned;
  }

  public sendErrorEmail(modalValue: { shouldSend: boolean; content: string }): void {
    this.displayModalError();
    if (modalValue.shouldSend) {
      this.structureService.sendMailOnStructureError(this.structure._id, modalValue.content).subscribe(() => {});
    }
  }

  public multipleWorkshop(): boolean {
    if (this.structure.categories.baseSkills.length + this.structure.categories.advancedSkills.length > 1) {
      return true;
    }
    return false;
  }

  public toggleBaseSkills(): void {
    this.showBaseSkills = !this.showBaseSkills;
  }

  public toggleAdvancedSkills(): void {
    this.showAdvancedSkills = !this.showAdvancedSkills;
  }

  public goToWebsite(): void {
    let url = this.structure.website;
    if (!url.startsWith('http')) {
      url = 'https://' + url;
    }
    window.open(url, '_blank');
  }

  public displayJobEmployer(profile: User): string {
    return new Utils().getJobEmployer(profile);
  }

  public displayTrainings(): boolean {
    if (this.structure.categoriesDisplay === undefined) return false;

    return (
      (this.structure.categoriesDisplay.age && this.structure.categoriesDisplay.age.length !== 0) ||
      (this.structure.categoriesDisplay.languageAndIlliteracy &&
        this.structure.categoriesDisplay.languageAndIlliteracy.length !== 0) ||
      (this.structure.categoriesDisplay.handicaps && this.structure.categoriesDisplay.handicaps.length !== 0) ||
      (this.structure.categoriesDisplay.publicOther && this.structure.categoriesDisplay.publicOther.length !== 0)
    );
  }
}
