import {
  FormControl,
  FormGroup,
  UntypedFormArray,
  UntypedFormControl,
  UntypedFormGroup,
  Validators,
} from '@angular/forms';
import { structureFormStep } from '../form/form-view/structure-form/structureFormStep.enum';
import { FiltersSteps } from '../form/orientation-form-view/enums/filtersSteps.enum';
import { HotlineMediationSteps } from '../form/orientation-form-view/enums/hotlineMediationSteps.enum';
import { MeetingRDVSSteps } from '../form/orientation-form-view/enums/MeetingRDVSSteps.enum';
import { NeedsType } from '../form/orientation-form-view/enums/needs.enum';
import { OnlineDemarchesCommonSteps } from '../form/orientation-form-view/enums/onlineDemarchesCommonSteps.enum';
import { PreferredLanguages } from '../form/orientation-form-view/enums/preferredLanguages.enum';
import { StructuresListSteps } from '../form/orientation-form-view/enums/structuresListSteps.enum';
import { FiltersForm } from '../form/orientation-form-view/interfaces/filtersForm.interface';
import { MeetingRDVSForm } from '../form/orientation-form-view/interfaces/meetingRDVSForm.interface';
import { OnlineDemarchesForm } from '../form/orientation-form-view/interfaces/onlineDemarchesForm.interface';
import { StructureOrientationForm } from '../form/orientation-form-view/interfaces/structureOrientationForm.interface';
import { StructureOrientator } from '../form/orientation-form-view/interfaces/structureOrientator.interface';
import { Module } from '../structure-list/models/module.model';
import { CustomRegExp } from './CustomRegExp';

export interface IStructureSummary {
  step: structureFormStep;
  name: string;
  inputs: Array<string> | null;
  hideForEdit?: boolean;
}
export interface INeedItem {
  title: string;
  hint: string;
  key: NeedsType | string;
}
export class OrientationUtils {
  public needsList: INeedItem[] = [
    {
      title: 'Accéder à du matériel numérique en libre service',
      hint: '(Wifi, ordinateur, imprimante, scanner…)',
      key: NeedsType.equipmentAccess,
    },
    {
      title: 'Acheter du matériel numérique à tarif solidaire',
      hint: '(Ordinateur, smartphone, clé 4G, forfait internet…)',
      key: NeedsType.equipmentBuy,
    },
    {
      title: 'Réaliser une démarche en ligne',
      hint: '(Démarche Caf, CPAM, Pôle emploi…)',
      key: NeedsType.onlineDemarch,
    },
    {
      title: 'Acquérir les compétences numériques de base',
      hint: '(Ordinateur et smartphone, mails, réseaux sociaux et bureautique...)',
      key: NeedsType.learnSkills,
    },
  ];

  public createOnlineDemarchesForm(): FormGroup<OnlineDemarchesForm> {
    return new FormGroup<OnlineDemarchesForm>({
      onlineDemarcheType: new FormControl<Module[]>([], Validators.required),
      accompanimentType: new UntypedFormControl(null, Validators.required),
    });
  }

  public createFiltersForm(): FormGroup<FiltersForm> {
    return new FormGroup<FiltersForm>({
      filters: new FormControl([], Validators.required),
    });
  }

  public createOnlineMediationForm(onlineDemarcheType: Module[]): UntypedFormGroup {
    return new UntypedFormGroup({
      filters: new FormControl<Module[]>(onlineDemarcheType, Validators.required),
      name: new UntypedFormControl('', [Validators.required, Validators.pattern(CustomRegExp.TEXT_WITHOUT_NUMBER)]),
      surname: new UntypedFormControl('', [Validators.required, Validators.pattern(CustomRegExp.TEXT_WITHOUT_NUMBER)]),
      phone: new UntypedFormControl('', [Validators.required, Validators.pattern(CustomRegExp.PHONE)]),
      dateSlot: new UntypedFormControl(null, Validators.required),
      preferredLanguage: new UntypedFormControl(PreferredLanguages.french, Validators.required),
    });
  }

  public createStructureOrientationForm(
    filters: Module[] = [],
    orientatorForm?: FormGroup<StructureOrientator>
  ): FormGroup<StructureOrientationForm> {
    return new FormGroup<StructureOrientationForm>({
      filters: new FormControl<Module[]>(filters, Validators.required),
      pmrAccess: new FormControl(null, Validators.required),
      address: new FormControl(''),
      structureChoice: new FormControl([], Validators.required),
      name: new FormControl('', [Validators.required, Validators.pattern(CustomRegExp.TEXT_WITHOUT_NUMBER)]),
      surname: new FormControl('', [Validators.required, Validators.pattern(CustomRegExp.TEXT_WITHOUT_NUMBER)]),
      comments: new FormControl('', [Validators.required]),
      structureOrientator: orientatorForm || this.createStructureOrientatorForm(),
    });
  }

  public createStructureOrientatorForm(orientator?: {
    structureName: string;
    structureMail: string;
    structurePhone: string;
  }): FormGroup<StructureOrientator> {
    return new FormGroup<StructureOrientator>({
      structureName: new FormControl(orientator?.structureName || '', Validators.required),
      structureMail: new FormControl(orientator?.structureMail || '', [Validators.pattern(CustomRegExp.EMAIL)]),
      structurePhone: new FormControl(orientator?.structurePhone || '', [Validators.pattern(CustomRegExp.PHONE)]),
    });
  }
  public createMeetingRDVSForm(
    onlineDemarcheType,
    orientatorForm?: FormGroup<StructureOrientator>
  ): FormGroup<MeetingRDVSForm> {
    return new FormGroup<MeetingRDVSForm>({
      structureOrientator: orientatorForm || this.createStructureOrientatorForm(),
      categories: new UntypedFormGroup(
        {
          age: new UntypedFormArray([], Validators.required),
          languageAndIlliteracy: new UntypedFormArray([]),
          publicOthers: new UntypedFormArray([]),
          handicaps: new UntypedFormArray([]),
        },
        Validators.required
      ),
      pmrAccess: new FormControl(null, Validators.required),
      digitalPass: new FormControl(null, Validators.required),
      address: new FormControl(''),
      onlineDemarcheType: new FormControl(onlineDemarcheType),
    });
  }
  /**
   *
   * @param pagesValidation
   * @param form
   * @param updatePageValid
   * @param step
   */
  public setValidationsOnlineDemarchesForm(
    pagesValidation: any[],
    form: UntypedFormGroup,
    updatePageValid: (isValid: boolean) => void,
    step: OnlineDemarchesCommonSteps
  ): void {
    pagesValidation[OnlineDemarchesCommonSteps.onlineDemarche] = {
      valid: form.get('onlineDemarcheType').value.length > 0,
    };
    pagesValidation[OnlineDemarchesCommonSteps.accompanimentType] = {
      valid: form.get('accompanimentType').value !== null,
    };
    updatePageValid(pagesValidation[step].valid);
  }

  /**
   *
   * @param pagesValidation
   * @param form
   * @param updatePageValid
   * @param step
   */
  public setValidationsOnlineMediationForm(
    pagesValidation: any[],
    form: UntypedFormGroup,
    updatePageValid: (isValid: boolean) => void,
    step: HotlineMediationSteps
  ): void {
    pagesValidation[HotlineMediationSteps.infoScreen] = {
      valid: true,
    };
    pagesValidation[HotlineMediationSteps.mediationBeneciaryInfo] = {
      valid: form.get('name').valid && form.get('surname').valid && form.get('phone').valid,
    };
    pagesValidation[HotlineMediationSteps.mediationHoursSelection] = {
      valid: form.get('dateSlot').value !== null,
    };
    pagesValidation[HotlineMediationSteps.mediationLanguageSelection] = {
      valid: true,
    };
    pagesValidation[HotlineMediationSteps.mediationRecap] = {
      valid: true,
    };
    updatePageValid(pagesValidation[step].valid);
  }

  /**
   *
   * @param pagesValidation
   * @param form
   * @param updatePageValid
   * @param step
   */
  public setValidationsStructuresForm(
    pagesValidation: any[],
    form: FormGroup<StructureOrientationForm>,
    updatePageValid: (isValid: boolean) => void,
    step: StructuresListSteps
  ): void {
    pagesValidation[StructuresListSteps.pmrAccess] = {
      valid: form.get('pmrAccess').valid,
    };
    pagesValidation[StructuresListSteps.address] = { valid: true };
    pagesValidation[StructuresListSteps.structureChoice] = {
      valid: form.get('structureChoice').valid,
    };
    pagesValidation[StructuresListSteps.structureOrientator] = {
      valid: form.get('structureOrientator').valid,
    };
    pagesValidation[StructuresListSteps.mediationBeneciaryInfo] = {
      valid: form.get('name').valid && form.get('surname').valid,
    };
    pagesValidation[StructuresListSteps.comments] = { valid: true };
    pagesValidation[StructuresListSteps.mediationRecap] = { valid: true };
    updatePageValid(pagesValidation[step].valid);
  }

  /**
   *
   * @param pagesValidation
   * @param form
   * @param updatePageValid
   * @param step
   */
  public setValidationsOnlineMeetingForm(
    pagesValidation: any[],
    form: FormGroup<MeetingRDVSForm>,
    updatePageValid: (isValid: boolean) => void,
    step: MeetingRDVSSteps
  ): void {
    pagesValidation[MeetingRDVSSteps.infoScreen] = {
      valid: true,
    };
    pagesValidation[MeetingRDVSSteps.structureOrientator] = {
      valid: form.get('structureOrientator').valid,
    };
    pagesValidation[MeetingRDVSSteps.publicTypeMandatory] = {
      valid: form.get('categories').valid && form.get('categories').get('age').valid,
    };
    pagesValidation[MeetingRDVSSteps.publicTypeOptional] = {
      valid: true,
    };
    pagesValidation[MeetingRDVSSteps.pmrAccess] = {
      valid: form.get('pmrAccess').valid,
    };
    pagesValidation[MeetingRDVSSteps.digitalPass] = {
      valid: true,
    };
    pagesValidation[MeetingRDVSSteps.location] = {
      valid: true,
    };

    updatePageValid(pagesValidation[step].valid);
  }

  /**
   *
   * @param pagesValidation
   * @param form
   * @param updatePageValid
   * @param step
   */
  public setValidationsEquipmentForm(
    pagesValidation: any[],
    form: FormGroup<FiltersForm>,
    updatePageValid: (isValid: boolean) => void,
    step: FiltersSteps
  ): void {
    pagesValidation[FiltersSteps.filterChoice] = {
      valid: form.get('filters').valid,
    };
    updatePageValid(pagesValidation[step].valid);
  }
}
